@@ -7,16 +7,16 @@ import numpy as np
 import pandas as pd
 import sciris as sc
 from collections import defaultdict
-from . import misc as cvm
 from . import defaults as cvd
+from . import misc as cvm
 from . import base as cvb
 from . import sim as cvs
-from . import plotting as cvplt
+from . import plotting as cvpl
 from .settings import options as cvo
 
 
 # Specify all externally visible functions this file defines
-__all__ = ['make_metapars', 'MultiSim', 'Scenarios', 'single_run', 'multi_run']
+__all__ = ['make_metapars', 'MultiSim', 'Scenarios', 'single_run', 'multi_run', 'parallel']
 
 
 
@@ -86,7 +86,7 @@ class MultiSim(cvb.FlexPretty):
         # Set properties
         self.sims      = sims
         self.base_sim  = base_sim
-        self.label     = label
+        self.label     = base_sim.label if (label is None and base_sim is not None) else label
         self.run_args  = sc.mergedicts(kwargs)
         self.results   = None
         self.which     = None # Whether the multisim is to be reduced, combined, etc.
@@ -162,6 +162,11 @@ class MultiSim(cvb.FlexPretty):
         else:
             sims = self.sims
 
+            # Handle missing labels
+            for s,sim in enumerate(sims):
+                if sim.label is None:
+                    sim.label = f'Sim {s}'
+
         # Run
         kwargs = sc.mergedicts(self.run_args, kwargs)
         self.sims = multi_run(sims, **kwargs)
@@ -175,6 +180,20 @@ class MultiSim(cvb.FlexPretty):
         return self
 
 
+    def _has_orig_sim(self):
+        ''' Helper method for determining if an original base sim is present '''
+        return hasattr(self, 'orig_base_sim')
+
+
+    def _rm_orig_sim(self, reset=False):
+        ''' Helper method for removing the original base sim, if present '''
+        if self._has_orig_sim():
+            if reset:
+                self.base_sim = self.orig_base_sim
+            delattr(self, 'orig_base_sim')
+        return
+
+
     def shrink(self, **kwargs):
         '''
         Not to be confused with reduce(), this shrinks each sim in the msim;
@@ -184,6 +203,7 @@ class MultiSim(cvb.FlexPretty):
             kwargs (dict): passed to sim.shrink() for each sim
         '''
         self.base_sim.shrink(**kwargs)
+        self._rm_orig_sim()
         for sim in self.sims:
             sim.shrink(**kwargs)
         return
@@ -191,9 +211,7 @@ class MultiSim(cvb.FlexPretty):
 
     def reset(self):
         ''' Undo a combine() or reduce() by resetting the base sim, which, and results '''
-        if hasattr(self, 'orig_base_sim'):
-            self.base_sim = self.orig_base_sim
-            delattr(self, 'orig_base_sim')
+        self._rm_orig_sim(reset=True)
         self.which = None
         self.results = None
         return
@@ -241,25 +259,25 @@ class MultiSim(cvb.FlexPretty):
         # Perform the statistics
         raw = {}
         mainkeys = reduced_sim.result_keys('main')
-        strainkeys = reduced_sim.result_keys('strain')
+        variantkeys = reduced_sim.result_keys('variant')
         for reskey in mainkeys:
             raw[reskey] = np.zeros((reduced_sim.npts, len(self.sims)))
             for s,sim in enumerate(self.sims):
                 vals = sim.results[reskey].values
                 raw[reskey][:, s] = vals
-        for reskey in strainkeys:
-            raw[reskey] = np.zeros((reduced_sim['n_strains'], reduced_sim.npts, len(self.sims)))
+        for reskey in variantkeys:
+            raw[reskey] = np.zeros((reduced_sim['n_variants'], reduced_sim.npts, len(self.sims)))
             for s,sim in enumerate(self.sims):
-                vals = sim.results['strain'][reskey].values
+                vals = sim.results['variant'][reskey].values
                 raw[reskey][:, :, s] = vals
 
-        for reskey in mainkeys + strainkeys:
+        for reskey in mainkeys + variantkeys:
             if reskey in mainkeys:
                 axis = 1
                 results = reduced_sim.results
             else:
                 axis = 2
-                results = reduced_sim.results['strain']
+                results = reduced_sim.results['variant']
             if use_mean:
                 r_mean = np.mean(raw[reskey], axis=axis)
                 r_std = np.std(raw[reskey], axis=axis)
@@ -500,8 +518,8 @@ class MultiSim(cvb.FlexPretty):
 
             # Plot
             for s,ind in enumerate(inds):
-                sim = self.sims[ind]
 
+                sim = self.sims[ind]
                 final_plot = (s == n_sims-1) # Check if this is the final plot
 
                 # Handle the legend and labels
@@ -529,9 +547,9 @@ class MultiSim(cvb.FlexPretty):
 
                 # Actually plot
                 merged_plot_args = sc.mergedicts({'alpha':alphas[s]}, plot_args) # Need a new variable to avoid overwriting
-                fig = sim.plot(fig=fig, to_plot=to_plot, colors=colors[s], labels=merged_labels, plot_args=merged_plot_args, show_args=merged_show_args, **kwargs)
+                fig = sim.plot(fig=fig, to_plot=('scens', to_plot), colors=colors[s], labels=merged_labels, plot_args=merged_plot_args, show_args=merged_show_args, **kwargs)
 
-        return fig
+        return cvpl.handle_show_return(fig=fig)
 
 
     def plot_result(self, key, colors=None, labels=None, *args, **kwargs):
@@ -551,7 +569,7 @@ class MultiSim(cvb.FlexPretty):
                 else:
                     kwargs['setylim'] = False
                 fig = sim.plot_result(key=key, fig=fig, color=colors[s], label=labels[s], *args, **kwargs)
-        return fig
+        return cvpl.handle_show_return(fig=fig)
 
 
     def plot_compare(self, t=-1, sim_inds=None, log_scale=True, **kwargs):
@@ -569,7 +587,7 @@ class MultiSim(cvb.FlexPretty):
             fig: Figure handle
         '''
         df = self.compare(t=t, sim_inds=sim_inds, output=True)
-        cvplt.plot_compare(df, log_scale=log_scale, **kwargs)
+        return cvpl.plot_compare(df, log_scale=log_scale, **kwargs)
 
 
     def save(self, filename=None, keep_people=False, **kwargs):
@@ -660,14 +678,16 @@ class MultiSim(cvb.FlexPretty):
             args = args[0] # A single list of MultiSims has been provided
 
         # Create the multisim from the base sim of the first argument
-        msim = MultiSim(base_sim=sc.dcp(args[0].base_sim))
+        msim = MultiSim(base_sim=sc.dcp(args[0].base_sim), label=args[0].label)
         msim.sims = []
         msim.chunks = [] # This is used to enable automatic splitting later
 
         # Handle different options for combining
         if base: # Only keep the base sims
             for i,ms in enumerate(args):
-                msim.sims.append(sc.dcp(ms.base_sim))
+                sim = sc.dcp(ms.base_sim)
+                sim.label = ms.label
+                msim.sims.append(sim)
                 msim.chunks.append([[i]])
         else: # Keep all the sims
             for ms in args:
@@ -688,7 +708,7 @@ class MultiSim(cvb.FlexPretty):
 
         Args:
             inds (list): a list of lists of indices, with each list turned into a MultiSim
-            chunks (int or list): if an int, split the MultiSim into chunks of that length; if a list return chunks of that many sims
+            chunks (int or list): if an int, split the MultiSim into that many chunks; if a list return chunks of that many sims
 
         Returns:
             A list of MultiSim objects
@@ -705,7 +725,7 @@ class MultiSim(cvb.FlexPretty):
             msim.run()
             m1, m2 = msim.split(inds=[[0,2,4], [1,3,5]])
             mlist1 = msim.split(chunks=[2,4]) # Equivalent to inds=[[0,1], [2,3,4,5]]
-            mlist2 = msim.split(chunks=3) # Equivalent to inds=[[0,1,2], [3,4,5]]
+            mlist2 = msim.split(chunks=2) # Equivalent to inds=[[0,1,2], [3,4,5]]
         '''
 
         # Process indices and chunks
@@ -824,11 +844,17 @@ class MultiSim(cvb.FlexPretty):
 
     def to_json(self, *args, **kwargs):
         ''' Shortcut for base_sim.to_json() '''
+        if not self.base_sim.results_ready: # pragma: no cover
+            errormsg = 'JSON export only available for reduced sim; please run msim.mean() or msim.median() first'
+            raise RuntimeError(errormsg)
         return self.base_sim.to_json(*args, **kwargs)
 
 
     def to_excel(self, *args, **kwargs):
         ''' Shortcut for base_sim.to_excel() '''
+        if not self.base_sim.results_ready: # pragma: no cover
+            errormsg = 'Excel export only available for reduced sim; please run msim.mean() or msim.median() first'
+            raise RuntimeError(errormsg)
         return self.base_sim.to_excel(*args, **kwargs)
 
 
@@ -886,13 +912,14 @@ class Scenarios(cvb.ParsObj):
         self.base_sim.update_pars(self.basepars)
         self.base_sim.validate_pars()
         if not self.base_sim.initialized:
-            self.base_sim.init_strains()
+            self.base_sim.init_variants()
             self.base_sim.init_immunity()
             self.base_sim.init_results()
 
         # Copy quantities from the base sim to the main object
-        self.npts = self.base_sim.npts
-        self.tvec = self.base_sim.tvec
+        self.npts       = self.base_sim.npts
+        self.tvec       = self.base_sim.tvec
+        self.datevec    = self.base_sim.datevec
         self['verbose'] = self.base_sim['verbose']
 
         # Create the results object; order is: results key, scenario, best/low/high
@@ -942,7 +969,7 @@ class Scenarios(cvb.ParsObj):
             return
 
         mainkeys   = self.result_keys('main')
-        strainkeys = self.result_keys('strain')
+        variantkeys = self.result_keys('variant')
 
         # Loop over scenarios
         for scenkey,scen in self.scenarios.items():
@@ -957,15 +984,13 @@ class Scenarios(cvb.ParsObj):
             # Create and run the simulations
             print_heading(f'Multirun for {scenkey}')
             scen_sim = sc.dcp(self.base_sim)
-            scen_sim.label = scenkey
+            scen_sim.scenkey = scenkey
+            scen_sim.label = scenname
+            scen_sim.scen = scen
 
-            scen_sim.update_pars(scenpars)  # Update the parameters, if provided
-            scen_sim.validate_pars()
-            if 'strains' in scenpars: # Process strains
-                scen_sim.init_strains()
-                scen_sim.init_immunity(create=True)
-            elif 'imm_pars' in scenpars: # Process immunity
-                scen_sim.init_immunity(create=True) # TODO: refactor
+            # Update the parameters, if provided, and re-initialize aspects of the simulation
+            scen_sim.update_pars(scenpars)
+            scen_sim.initialized = False # Ensure it gets re-initialized
 
             run_args = dict(n_runs=self['n_runs'], noise=self['noise'], noisepar=self['noisepar'], keep_people=keep_people, verbose=verbose)
             if debug:
@@ -977,28 +1002,28 @@ class Scenarios(cvb.ParsObj):
 
             # Process the simulations
             print_heading(f'Processing {scenkey}')
-            ns = scen_sims[0]['n_strains'] # Get number of strains
+            ns = scen_sims[0]['n_variants'] # Get number of variants
             scenraw = {}
             for reskey in mainkeys:
                 scenraw[reskey] = np.zeros((self.npts, len(scen_sims)))
                 for s,sim in enumerate(scen_sims):
                     scenraw[reskey][:,s] = sim.results[reskey].values
-            for reskey in strainkeys:
+            for reskey in variantkeys:
                 scenraw[reskey] = np.zeros((ns, self.npts, len(scen_sims)))
                 for s,sim in enumerate(scen_sims):
-                    scenraw[reskey][:,:,s] = sim.results['strain'][reskey].values
+                    scenraw[reskey][:,:,s] = sim.results['variant'][reskey].values
 
             scenres = sc.objdict()
             scenres.best = {}
             scenres.low = {}
             scenres.high = {}
-            for reskey in mainkeys + strainkeys:
+            for reskey in mainkeys + variantkeys:
                 axis = 1 if reskey in mainkeys else 2
                 scenres.best[reskey] = np.quantile(scenraw[reskey], q=0.5, axis=axis) # Changed from median to mean for smoother plots
                 scenres.low[reskey]  = np.quantile(scenraw[reskey], q=self['quantiles']['low'], axis=axis)
                 scenres.high[reskey] = np.quantile(scenraw[reskey], q=self['quantiles']['high'], axis=axis)
 
-            for reskey in mainkeys + strainkeys:
+            for reskey in mainkeys + variantkeys:
                 self.results[reskey][scenkey]['name'] = scenname
                 for blh in ['best', 'low', 'high']:
                     self.results[reskey][scenkey][blh] = scenres[blh][reskey]
@@ -1041,14 +1066,14 @@ class Scenarios(cvb.ParsObj):
 
         # Compute dataframe
         x = defaultdict(dict)
-        strainkeys = self.result_keys('strain')
+        variantkeys = self.result_keys('variant')
         for scenkey in self.scenarios.keys():
             for reskey in self.result_keys():
-                if reskey in strainkeys:
-                    for strain in range(self.base_sim['n_strains']):
-                        val = self.results[reskey][scenkey].best[strain, day] # Only prints results for infections by first strain
-                        strainkey = reskey + str(strain) # Add strain number to the summary output
-                        x[scenkey][strainkey] = int(val)
+                if reskey in variantkeys:
+                    for variant in range(self.base_sim['n_variants']):
+                        val = self.results[reskey][scenkey].best[variant, day] # Only prints results for infections by first variant
+                        variantkey = reskey + str(variant) # Add variant number to the summary output
+                        x[scenkey][variantkey] = int(val)
                 else:
                     val = self.results[reskey][scenkey].best[day]
                     if reskey not in ['r_eff', 'doubling_time']:
@@ -1078,8 +1103,7 @@ class Scenarios(cvb.ParsObj):
             scens.run()
             scens.plot()
         '''
-        fig = cvplt.plot_scens(scens=self, *args, **kwargs)
-        return fig
+        return cvpl.plot_scens(scens=self, *args, **kwargs)
 
 
     def to_json(self, filename=None, tostring=True, indent=2, verbose=False, *args, **kwargs):
@@ -1123,7 +1147,7 @@ class Scenarios(cvb.ParsObj):
         spreadsheet = sc.Spreadsheet()
         spreadsheet.freshbytes()
         with pd.ExcelWriter(spreadsheet.bytes, engine='xlsxwriter') as writer:
-            for key in self.result_keys('main'): # Multidimensional strain keys can't be exported
+            for key in self.result_keys('main'): # Multidimensional variant keys can't be exported
                 result_df = pd.DataFrame.from_dict(sc.flattendict(self.results[key], sep='_'))
                 result_df.to_excel(writer, sheet_name=key)
         spreadsheet.load()
@@ -1170,8 +1194,8 @@ class Scenarios(cvb.ParsObj):
         if keep_sims or keep_people:
             if keep_people:
                 if not obj._kept_people:
-                    print('Warning: there are no people because they were not saved during the run. '
-                          'If you want people, please rerun with keep_people=True.')
+                    warnmsg = 'Warning: there are no people because they were not saved during the run. If you want people, please rerun with keep_people=True.'
+                    cvm.warn(warnmsg)
                 obj.sims = sims # Just restore the object in full
                 print('Note: saving people, which may produce a large file!')
             else:
@@ -1334,7 +1358,7 @@ def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, keep_people=Fa
         verbose = sim['verbose']
 
     if not sim.label:
-        sim.label = f'Sim {ind:d}'
+        sim.label = f'Sim {ind}'
 
     if reseed:
         sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
@@ -1379,7 +1403,7 @@ def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, keep_people=Fa
     return sim
 
 
-def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=None, combine=False, keep_people=None, run_args=None, sim_args=None, par_args=None, do_run=True, parallel=True, n_cpus=None, verbose=None, **kwargs):
+def multi_run(sim, n_runs=4, reseed=None, noise=0.0, noisepar=None, iterpars=None, combine=False, keep_people=None, run_args=None, sim_args=None, par_args=None, do_run=True, parallel=True, n_cpus=None, verbose=None, **kwargs):
     '''
     For running multiple runs in parallel. If the first argument is a list of sims,
     exactly these will be run and most other arguments will be ignored.
@@ -1387,7 +1411,7 @@ def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=Non
     Args:
         sim         (Sim)   : the sim instance to be run, or a list of sims.
         n_runs      (int)   : the number of parallel runs
-        reseed      (bool)  : whether or not to generate a fresh seed for each run
+        reseed      (bool)  : whether or not to generate a fresh seed for each run (default: true for single, false for list of sims)
         noise       (float) : the amount of noise to add to each run
         noisepar    (str)   : the name of the parameter to add noise to
         iterpars    (dict)  : any other parameters to iterate over the runs; see sc.parallelize() for syntax
@@ -1430,13 +1454,15 @@ def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=Non
                 n_runs = new_n
 
     # Run the sims
-    if isinstance(sim, cvs.Sim): # Normal case: one sim
-        iterkwargs = {'ind':np.arange(n_runs)}
+    if isinstance(sim, cvs.Sim): # One sim
+        if reseed is None: reseed = True
+        iterkwargs = dict(ind=np.arange(n_runs))
         iterkwargs.update(iterpars)
         kwargs = dict(sim=sim, reseed=reseed, noise=noise, noisepar=noisepar, verbose=verbose, keep_people=keep_people, sim_args=sim_args, run_args=run_args, do_run=do_run)
     elif isinstance(sim, list): # List of sims
-        iterkwargs = {'sim':sim}
-        kwargs = dict(verbose=verbose, keep_people=keep_people, sim_args=sim_args, run_args=run_args, do_run=do_run)
+        if reseed is None: reseed = False
+        iterkwargs = dict(sim=sim, ind=np.arange(len(sim)))
+        kwargs = dict(reseed=reseed, verbose=verbose, keep_people=keep_people, sim_args=sim_args, run_args=run_args, do_run=do_run)
     else:
         errormsg = f'Must be Sim object or list, not {type(sim)}'
         raise TypeError(errormsg)
@@ -1475,3 +1501,28 @@ Alternatively, to run without multiprocessing, set parallel=False.
             sims.append(sim)
 
     return sims
+
+
+def parallel(*args, **kwargs):
+    '''
+    A shortcut to ``cv.MultiSim()``, allowing the quick running of multiple simulations
+    at once.
+
+    Args:
+        args (list): The simulations to run
+        kwargs (dict): passed to multi_run()
+
+    Returns:
+        A run MultiSim object.
+
+    **Examples**::
+
+        s1 = cv.Sim(beta=0.01, label='Low')
+        s2 = cv.Sim(beta=0.02, label='High')
+        cv.parallel(s1, s2).plot()
+        msim = cv.parallel([s1, s2], keep_people=True)
+
+    New in version 3.1.1.
+    '''
+    sims = sc.mergelists(*args)
+    return MultiSim(sims=sims).run(**kwargs)
